source(g_currentModDirectory .. "scripts/overrideScripts.lua");


-- register new configuration types
if ConfigurationUtil.configurations["balersExtension"] == nil then
    ConfigurationUtil.registerConfigurationType("balersExtension", g_i18n:getText("l10n_balersExtension"), nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION);
end;

BalerExtension = {};

function BalerExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Baler, specializations)
end;

function BalerExtension:preLoad(savegame)
end;

function BalerExtension:load(savegame)
    self.getIsPlayerInRange = BalerExtension.getIsPlayerInRange;
    self.applyMaterialAging = BalerExtension.applyMaterialAging;
    self.dropBale = Utils.overwrittenFunction(self.dropBale, BalerExtension.dropBale);
    
    self.coverRangeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.cover#checkNode"));
    self.coverRangeDistance = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.cover#checkDistance"), 1);
    
    BalerExtension.loadAttacherJoints(self, self.xmlFile, self.configurations["balersExtension"]);
    BalerExtension.loadColor(self, self.xmlFile, self.configurations["balersExtension"]);
    
    ObjectChangeUtil.updateObjectChanges(self.xmlFile, "vehicle.balersExtensionConfigurations.balersExtensionConfiguration", self.configurations["balersExtension"], self.components, self);
end;

function BalerExtension:postLoad(savegame)
    BalerExtension.loadBaleAnimations(self, self.xmlFile, self.configurations["balersExtension"]);
end;

function BalerExtension:loadBaleAnimations(xmlFile, configId)
    local key, configId = Vehicle.getXMLConfigurationKey(xmlFile, configId, "vehicle.balersExtensionConfigurations.balersExtensionConfiguration", "vehicle", "balersExtension");

    local firstBaleMarker = getXMLFloat(self.xmlFile, key .. ".baleAnimation#firstBaleMarker");
    if firstBaleMarker ~= nil then
        local baleAnimCurve = AnimCurve:new(linearInterpolatorN);
        local keyI = 0;
        while true do
            local key = string.format(key .. ".baleAnimation.key(%d)", keyI);
            local t = getXMLFloat(self.xmlFile, key.."#time");
            local x,y,z = Utils.getVectorFromString(getXMLString(self.xmlFile, key.."#pos"));
            if x == nil or y == nil or z == nil then
                break;
            end;
            local rx, ry, rz = Utils.getVectorFromString(getXMLString(self.xmlFile, key.."#rot"));
            rx = math.rad(Utils.getNoNil(rx, 0));
            ry = math.rad(Utils.getNoNil(ry, 0));
            rz = math.rad(Utils.getNoNil(rz, 0));
            baleAnimCurve:addKeyframe({ v={x, y, z, rx, ry, rz}, time = t});
            keyI = keyI + 1;
        end;
        if keyI > 0 then
            self.baler.baleAnimCurve = baleAnimCurve;
            self.baler.firstBaleMarker = firstBaleMarker;
        end;
    end;
    
    self.baler.baleAnimRoot, self.baler.baleAnimRootComponent = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".baleAnimation#node"));
    if self.baler.baleAnimRoot == nil then
        self.baler.baleAnimRoot = self.components[1].node;
        self.baler.baleAnimRootComponent = self.components[1].node;
    end;
        
    self.baler.dummyBale.scaleNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".baleAnimation#scaleNode"));
    self.baler.dummyBale.baleNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".baleAnimation#baleNode"));
    
    self.baleThrowerVelocity = Utils.getNoNil(getXMLFloat(self.xmlFile, key .. ".baleAnimation#baleThrowerVelocity"), 0);
end;

function BalerExtension:loadAttacherJoints(xmlFile, configId)
    local key, configId = Vehicle.getXMLConfigurationKey(xmlFile, configId, "vehicle.balersExtensionConfigurations.balersExtensionConfiguration", "vehicle", "balersExtension");
    if self.attacherJoints ~= nil then
        local i = 0;
        while true do
            local baseName = string.format(key .. ".attacherJoints.attacherJoint(%d)", i);
            if not hasXMLProperty(xmlFile, baseName) then
                break;
            end;
            
            local attacherJoint = {};
            if self:loadAttacherJointFromXML(attacherJoint, xmlFile, baseName, i) then
                table.insert(self.attacherJoints, attacherJoint);
            end;
            i = i + 1;
        end;
    end;
end;

function BalerExtension:loadColor(xmlFile, configId)
    local key, configId = Vehicle.getXMLConfigurationKey(xmlFile, configId, "vehicle.balersExtensionConfigurations.balersExtensionConfiguration", "vehicle", "balersExtension");
    
    local colorId = self.configurations["baseColor"] - 1;
    local colorBase = string.format(key .. ".baseColorConfigurations.baseColorConfiguration(%d)", colorId)
    local color = Utils.getVectorNFromString(getXMLString(xmlFile, colorBase .. "#color"), 4);    
    
    if color ~= nil then
        local r,g,b,a = unpack(color);
        local i = 0;
        while true do
            local colorKey = string.format(key .. ".baseColorConfigurations.colorNode(%d)", i)
            if not hasXMLProperty(xmlFile, colorKey) then
                break;
            end;

            local node = Utils.indexToObject(self.components, getXMLString(xmlFile, colorKey .. "#node"));
            if node ~= nil then
                if getHasClassId(node, ClassIds.SHAPE) then
                    if Utils.getNoNil(getXMLBool(xmlFile, colorKey .. "#recursive"), false) then
                        Utils.setShaderParameterRec(node, "colorScale", r, g, b, a);
                    else
                        setShaderParameter(node, "colorScale", r, g, b, a, false);
                    end;
                else
                    print("Warning: Could not set vehicle color to '"..getName(node).."' because node is not a shape!");
                end;
            end;
            i = i + 1;
        end;
    end;
end;

function BalerExtension:applyMaterialAging(configFileName)
    local xmlFile = loadXMLFile("TempConfig", configFileName);
    local i = 0
    while true do
        local key = string.format("vehicle.balersExtensionConfigurations.machineAging.material(%d)", i)
        if not hasXMLProperty(xmlFile, key) then
            break;
        end;
        
        local baseMaterialNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#node"));
        local refMaterialNode = Utils.indexToObject(self.components, getXMLString(xmlFile, key .. "#refNode"));
        if baseMaterialNode ~= nil and refMaterialNode ~= nil then
            local oldMaterial = getMaterial(baseMaterialNode, 0)
            local newMaterial = getMaterial(refMaterialNode, 0)
            for _, component in pairs(self.components) do
                self:replaceMaterialRec(component.node, oldMaterial, newMaterial);
            end;
        end;
        i = i + 1;
    end;
    
    BalerExtension.loadColor(self, xmlFile, self.configurations["balersExtension"]);
end;

function BalerExtension:update(dt)
    if self.isClient then
        if self.coverAnimation ~= nil and self.coverRangeIndex ~= nil then
            if self:getIsPlayerInRange(self.coverRangeIndex, self.coverRangeDistance) then
                if self:getIsToggleCoverAllowed() then
                    if self.isCoverOpen then
                        g_currentMission:addHelpButtonText(g_i18n:getText("action_closeCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
                    else
                        g_currentMission:addHelpButtonText(g_i18n:getText("action_openCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
                    end;
                    
                    if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
                        self:setCoverState(not self.isCoverOpen);
                    end;
                end;
            end;
        end;
        
        local operatingTime = self.operatingTime / (1000*60*60);
        if not self.materialAging and operatingTime > 100 then
            self.materialAging = true;
            self:applyMaterialAging(self.configFileName);
        end;
    end;
    
    if self.isServer then
        if self.attachedImplements ~= nil then
            for _,implement in pairs(self.attachedImplements) do
                if implement.object ~= nil then
                    local vehicle = implement.object;
                    if vehicle.isAutoBaleLoader then
                        if vehicle.currentBaleType ~= nil then
                            if vehicle.currentBaleTypeId == nil then
                                for i, baleType in pairs(vehicle.allowedBaleTypes) do
                                    if baleType == vehicle.currentBaleType then
                                        vehicle.currentBaleTypeId = i;
                                        break;
                                    end;
                                end;
                            end;
                            
                            local object = self.baler.baleTypes[self.baler.currentBaleTypeId];
                            local baleType = vehicle.allowedBaleTypes[vehicle.currentBaleTypeId];
                            
                            if object.width ~= nil and object.height ~= nil and object.length ~= nil and
                               object.width >= baleType.minBaleWidth and object.width <= baleType.maxBaleWidth and
                               object.height >= baleType.minBaleHeight and object.height <= baleType.maxBaleHeight and
                               object.length >= baleType.minBaleLength and object.length <= baleType.maxBaleLength
                            then
                                if table.getn(vehicle.currentBaleType.balePlaces.bales) >= vehicle.currentBaleType.capacity then
                                    self:setIsTurnedOn(false);
                                end;
                            else
                                self:setIsTurnedOn(false);
                            end;
                        else
                            vehicle.currentBaleTypeId = nil;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

function BalerExtension:updateTick(dt)
end;

function BalerExtension:readStream(streamId, connection)
end;

function BalerExtension:writeStream(streamId, connection)
end;

function BalerExtension:delete()
end;

function BalerExtension:draw()
end;

function BalerExtension:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BalerExtension:keyEvent(unicode, sym, modifier, isDown)
end;

function BalerExtension:getIsPlayerInRange(index, minDistance)
    if g_currentMission.player ~= nil then
        local rx, ry, rz = getWorldTranslation(g_currentMission.player.rootNode);
        local ox, oy, oz = getWorldTranslation(index);
        local distance = Utils.vector3Length(rx-ox, ry-oy, rz-oz);

        if distance <= minDistance then
            return true;
        end;
    end;
        
    return false;
end;

function BalerExtension:dropBale(superFunc, baleIndex)
    if superFunc ~= nil then
        local bale = self.baler.bales[baleIndex]

        if self.isServer then
            local baleObject

            if bale.baleJointIndex ~= nil then
                baleObject = bale.baleObject
                removeJoint(bale.baleJointIndex)
                delete(bale.baleJointNode)
                g_currentMission:addItemToSave(bale.baleObject)
            else
                baleObject = Bale:new(self.isServer, self.isClient)
                local x,y,z = getWorldTranslation(bale.id)
                local rx,ry,rz = getWorldRotation(bale.id)
                baleObject:load(bale.filename, x,y,z,rx,ry,rz, bale.fillLevel)
                baleObject:register()
                delete(bale.id)
            end

            if (not self.hasBaleWrapper or self.moveBaleToWrapper == nil) and baleObject.nodeId ~= nil then
                -- release bale if there's no bale wrapper
                if self.baleThrowerVelocity ~= nil and self.baleThrowerVelocity > 0 then
                    local dx,dy,dz = localDirectionToWorld(baleObject.nodeId, 0,0,-1);
                    local vx = dx * self.baleThrowerVelocity;
                    local vy = dy * self.baleThrowerVelocity;
                    local vz = dz * self.baleThrowerVelocity;
                    setLinearVelocity(baleObject.nodeId, vx,vy,vz)
                else
                    local x,y,z = getWorldTranslation(baleObject.nodeId)
                    local vx,vy,vz = getVelocityAtWorldPos(self.baler.baleAnimRootComponent, x,y,z)
                    setLinearVelocity(baleObject.nodeId, vx,vy,vz)
                end;
            elseif self.moveBaleToWrapper ~= nil then
                -- move bale to wrapper
                self:moveBaleToWrapper(baleObject)
            end
        end

        Utils.releaseSharedI3DFile(bale.filename, nil, true)
        table.remove(self.baler.bales, baleIndex)

        g_currentMission.missionStats:updateStats("baleCount", 1)
    end;
end;