PowerTakeOffFix = {};

function PowerTakeOffFix.prerequisitesPresent(specializations)
    return true;
end;

function PowerTakeOffFix:load(savegame)
end;

function PowerTakeOffFix:postLoad(savegame)
    if FS17_manualAttaching ~= nil and g_currentMission.callbackManualAttaching ~= nil then
        if not g_currentMission.callbackManualAttaching.isFixedPTO then
            g_currentMission.callbackManualAttaching.isFixedPTO = true;
            g_currentMission.callbackManualAttaching.detachImplement = PowerTakeOffFix.detachImplement;
            g_currentMission.callbackManualAttaching.attachPowerTakeOff = PowerTakeOffFix.attachPowerTakeOff;
            g_currentMission.callbackManualAttaching.detachPowerTakeOff = PowerTakeOffFix.detachPowerTakeOff;
        end;
    end;
end;

function PowerTakeOffFix:update(dt)
end;

function PowerTakeOffFix:updateTick(dt)
end;

function PowerTakeOffFix:readStream(streamId, connection)
end;

function PowerTakeOffFix:writeStream(streamId, connection)
end;

function PowerTakeOffFix:delete()
end;

function PowerTakeOffFix:draw()
end;

function PowerTakeOffFix:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PowerTakeOffFix:keyEvent(unicode, sym, modifier, isDown)
end;


-- fix scripts of ManualAttaching
function PowerTakeOffFix:detachImplement(vehicle, object, implementIndex, force)
    if vehicle ~= nil and object ~= nil then
        local implement = vehicle.attachedImplements[implementIndex]
        local jointDesc = vehicle.attacherJoints[implement.jointDescIndex]
        
        if g_currentMission.callbackManualAttaching:isManual(vehicle, jointDesc) or force then
            -- (Saty MODS) add check if implement is "allowsLowering"
            if jointDesc.allowsLowering and implement.object.attacherJoint.allowsLowering then
                if not jointDesc.moveDown and not (jointDesc.jointType == AttacherJoints.jointTypeNameToInt['attachableFrontloader']) then
                    if not g_currentMission.callbackManualAttaching:getDoesNotNeedJointMovedown(jointDesc) then                        
                        g_currentMission.callbackManualAttaching:showWarning(g_currentMission.callbackManualAttaching.message.lowerWarning, object)
                        g_currentMission:enableHudIcon('detachingNotAllowed', 6, self.detachingNotAllowedTime)
                        
                        return false
                    end
                end
            end
            
            if self:scopeAllowsDetaching(implement.object, jointDesc) then
                -- handle TurnOnVehicle exceptions
                if object.activateTankOnLowering ~= nil and object.activateTankOnLowering then
                    if object.setIsTurnedOn ~= nil then
                        object:setIsTurnedOn(false)
                    end                    
                end

                if (vehicle.getIsTurnedOn ~= nil and vehicle:getIsTurnedOn()) or (object.getIsTurnedOn ~= nil and object:getIsTurnedOn()) then
                    g_currentMission.callbackManualAttaching:showWarning(g_currentMission.callbackManualAttaching.message.turnOffWarning, object)
                    g_currentMission:enableHudIcon('detachingNotAllowed', 6, self.detachingNotAllowedTime)
                    
                    return false
                end
            else
                return false
            end
                
            vehicle:detachImplementByObject(object)
            self:playSound(vehicle, jointDesc, self.currentSoundPlayer)
        end
    end
    
    return true
end

function PowerTakeOffFix:attachPowerTakeOff(vehicle, object, noEventSend)
    FS17_manualAttaching.ManualAttachingPTOEvent.sendEvent(vehicle, object, true, noEventSend)
    
    local implement = vehicle:getImplementByObject(object)

    if implement ~= nil then
        local jointDesc = vehicle.attacherJoints[implement.jointDescIndex]
        local object = implement.object

        local attachPto = function(object, jointDesc, type, overwriteOutput)
            if not jointDesc[('%sActive'):format(type)] then
                local input = object[('%sInput'):format(type)]
                local outputJointDesc = overwriteOutput ~= '' and jointDesc[('%sOutput'):format(overwriteOutput)] or jointDesc[('%sOutput'):format(type)]
                
                if input ~= nil then
                    local isActive = false
                    
                    if input.rootNode ~= nil then
                        if input.isNonScalable ~= nil and input.isNonScalable then
                            if object.attachMovingPowerTakeOff ~= nil then
                                object:attachMovingPowerTakeOff(object, input, outputJointDesc)                                
                                isActive = true
                            end
                        else 
                            link(outputJointDesc.node, input.rootNode)
                            link(input.node, input.attachNode)

                            if object.addWashableNode ~= nil then
                                object:addWashableNode(input.rootNode)
                                object:addWashableNode(input.attachNode)
                                object:addWashableNode(input.dirAndScaleNode)
                                if input.translatingPart ~= nil then
                                    object:addWashableNode(input.translatingPart);
                                end;
                                object:setDirtAmount(object:getDirtAmount(), true)
                            end
                            
                            isActive = true
                        end
                    else
                        if outputJointDesc ~= nil then
                            link(outputJointDesc.node, outputJointDesc.rootNode)
                            link(input.node, outputJointDesc.attachNode)

                            if object.addWashableNode ~= nil then
                                object:addWashableNode(outputJointDesc.rootNode)
                                object:addWashableNode(outputJointDesc.attachNode)
                                object:addWashableNode(outputJointDesc.dirAndScaleNode)
                                object:setDirtAmount(object:getDirtAmount(), true)
                            end
                            
                            isActive = true
                        end
                    end
            
                    jointDesc[('%sActive'):format(type)] = isActive
                    -- Only works if tool has powerConsumer spec, tool also needs neededPtoPower and ptoRpm to be > 0.
                    if isActive then
                        jointDesc.canTurnOnImplement = jointDesc.canTurnOnImplementBackUp
                    end
                end            
            end
        end
        
        for type, overwrite in pairs(g_currentMission.callbackManualAttaching.ptoTypes) do
            attachPto(object, jointDesc, type, overwrite)
        end
        
        vehicle:updatePowerTakeoff(implement, 0, 'pto')
        vehicle:updatePowerTakeoff(implement, 0, 'pto2')
    end
end

function PowerTakeOffFix:detachPowerTakeOff(vehicle, object, noEventSend)
    FS17_manualAttaching.ManualAttachingPTOEvent.sendEvent(vehicle, object, false, noEventSend)
    
    local implement = vehicle:getImplementByObject(object)

    if implement ~= nil then
        local jointDesc = vehicle.attacherJoints[implement.jointDescIndex]
        local object = implement.object
        
        local detachPto = function(object, jointDesc, type, overwriteOutput)
            -- Only works if tool has powerConsumer spec, tool also needs neededPtoPower and ptoRpm to be > 0.
            if jointDesc.canTurnOnImplementBackUp == nil then
                jointDesc.canTurnOnImplementBackUp = jointDesc.canTurnOnImplement
            end
            
            local overwriteType = overwriteOutput ~= '' and overwriteOutput or type
            
            if jointDesc[('%sActive'):format(type)] then
                local input = object[('%sInput'):format(type)]
                local outputJointDesc = overwriteOutput ~= '' and jointDesc[('%sOutput'):format(overwriteOutput)] or jointDesc[('%sOutput'):format(type)]
                local isActive = true
                
                if input ~= nil and input.rootNode ~= nil then
                    if input.isNonScalable ~= nil and input.isNonScalable then
                        if object.detachMovingPowerTakeOff ~= nil then
                            object:detachMovingPowerTakeOff(object, input)                            
                            isActive = false
                        end
                    else
                        unlink(input.rootNode)
                        unlink(input.attachNode)

                        if object.removeWashableNode ~= nil then
                            object:removeWashableNode(input.rootNode)
                            object:removeWashableNode(input.attachNode)
                            object:removeWashableNode(input.dirAndScaleNode)
                            if input.translatingPart ~= nil then
                                object:removeWashableNode(input.translatingPart);
                            end;
                        end                        
                    
                        isActive = false
                    end
                else
                    unlink(outputJointDesc.rootNode)
                    unlink(outputJointDesc.attachNode)

                    if object.removeWashableNode ~= nil then
                        object:removeWashableNode(outputJointDesc.rootNode)
                        object:removeWashableNode(outputJointDesc.attachNode)
                        object:removeWashableNode(outputJointDesc.dirAndScaleNode)
                    end
                    
                    isActive = false
                end
            end
            
            jointDesc[('%sActive'):format(type)] = isActive
            
            if not isActive then
                jointDesc.canTurnOnImplement = false
            end
        end
        
        for type, overwrite in pairs(g_currentMission.callbackManualAttaching.ptoTypes) do
            detachPto(object, jointDesc, type, overwrite)
        end
    end
end


-- fix spripts of game
if not BaseMission.isFixedPTO then
    BaseMission.isFixedPTO = true;

    function Attachable:loadInputPowerTakeoff(superFunc, xmlFile, baseName, entry, name)
        if superFunc ~= nil then
            superFunc(self, xmlFile, baseName, entry, name);
        end;

        if name == "pto" then
            Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, baseName.."#indexPTO", baseName.."#ptoInputNode")
            Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, baseName.."#rotSpeedPTO", baseName.."#ptoRotSpeed")
            Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, baseName.."#aboveAttacherPTO", baseName.."#ptoAboveAttacher")
            Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, baseName.."#sizePTO", baseName.."#ptoSize")
            Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, baseName.."#overrideFilenamePTO", baseName.."#ptoOverrideFilename")
        end;

        local xmlKey = baseName .. "#"..name;
        local ptoInputNode = Utils.indexToObject(self.components, getXMLString(xmlFile, xmlKey.."InputNode"));
        if ptoInputNode ~= nil then
            if entry[name.."Input"] == nil then
                entry[name.."Input"] = {};
            end;

            entry[name.."Input"].node = ptoInputNode;
            entry[name.."Input"].aboveAttacher = Utils.getNoNil(getXMLBool(xmlFile, xmlKey.."AboveAttacher"), true);
            entry[name.."Input"].size = Utils.getNoNil(getXMLFloat(xmlFile, xmlKey.."Size"), 0.09);
            
            entry[name.."Input"].haveTranslatingPart = Utils.getNoNil(getXMLBool(xmlFile, xmlKey.."HaveTranslatingPart"), false);

            local rotSpeed = math.rad(Utils.getNoNil(getXMLFloat(xmlFile, xmlKey.."RotSpeed"), 0)*0.001);
            if math.abs(rotSpeed) < 0.00001 then
                rotSpeed = 0;
            end;
            entry[name.."Input"].rotSpeed = rotSpeed;

            local filename = getXMLString(xmlFile, xmlKey .. "OverrideFilename");
            if filename ~= nil then
                local i3dNode = Utils.loadSharedI3DFile(filename, self.baseDirectory, false, false, false);
                if i3dNode ~= 0 then
                    if entry[name.."Input"].haveTranslatingPart then
                        local rootNode = getChildAt(i3dNode, 0);
                        unlink(rootNode);
                        delete(i3dNode);
                        --setTranslation(rootNode, 0,0,0);
                        local dirAndScaleNode = getChildAt(rootNode, 0);
                        local translatingPart = getChildAt(dirAndScaleNode, 0);
                        local attachNode = getChildAt(translatingPart, 0);
                        local attachRefNode = getChildAt(attachNode, 0);

                        local _,_,baseDistance = getTranslation(attachNode);
                        unlink(attachNode);
                        local ax, ay, az = getTranslation(attachRefNode);
                        setTranslation(attachNode, -ax, -ay, -az);

                        entry[name.."Input"].rootNode = rootNode;
                        entry[name.."Input"].dirAndScaleNode = dirAndScaleNode;
                        entry[name.."Input"].translatingPart = translatingPart;
                        entry[name.."Input"].attachNode = attachNode;
                        entry[name.."Input"].baseDistance = baseDistance;
                    else
                        local rootNode = getChildAt(i3dNode, 0);
                        unlink(rootNode);
                        delete(i3dNode);
                        setTranslation(rootNode, 0,0,0);
                        local dirAndScaleNode = getChildAt(rootNode, 0);
                        local attachNode = getChildAt(dirAndScaleNode, 0);
                        local attachRefNode = getChildAt(attachNode, 0);

                        local _,_,baseDistance = getTranslation(attachNode);
                        unlink(attachNode);
                        local ax, ay, az = getTranslation(attachRefNode);
                        setTranslation(attachNode, -ax, -ay, -az);

                        entry[name.."Input"].rootNode = rootNode;
                        entry[name.."Input"].dirAndScaleNode = dirAndScaleNode;
                        entry[name.."Input"].attachNode = attachNode;
                        entry[name.."Input"].baseDistance = baseDistance;
                    end;
                end;
            end;
        end;
    end;

    AttachImplementPrependedFunction = function(self, superFunc, object, inputJointDescIndex, jointDescIndex, noEventSend, index, startLowered, noSmoothAttach)
        if object.addWashableNode ~= nil then
            object.addWashableNodeBackup = object.addWashableNode;
            object.addWashableNode = nil;
        end;
    end;
    AttacherJoints.attachImplement = Utils.prependedFunction(AttacherJoints.attachImplement, AttachImplementPrependedFunction);

    AttachImplementAppendedFunction = function(self, superFunc, object, inputJointDescIndex, jointDescIndex, noEventSend, index, startLowered, noSmoothAttach)
        if object.addWashableNodeBackup ~= nil then
            object.addWashableNode = object.addWashableNodeBackup;
            object.addWashableNodeBackup = nil;
            
            local jointDesc = self.attacherJoints[jointDescIndex];
            
            if jointDesc["ptoActive"] then
                local input = object["ptoInput"];
                local jointDescOutput = jointDesc["ptoOutput"];
                if input ~= nil and jointDescOutput ~= nil then
                    object:addWashableNode(input.rootNode);
                    object:addWashableNode(input.attachNode);
                    object:addWashableNode(input.dirAndScaleNode);
                    if input.translatingPart ~= nil then
                        object:addWashableNode(input.translatingPart);
                    end;
                    object:setDirtAmount(object:getDirtAmount(), true);
                end;
            end;
                
            if jointDesc["pto2Active"] then
                local input = object["pto2Input"];
                local jointDescOutput = jointDesc["pto2Output"];
                if input ~= nil and jointDescOutput ~= nil then
                    object:addWashableNode(input.rootNode);
                    object:addWashableNode(input.attachNode);
                    object:addWashableNode(input.dirAndScaleNode);
                    if input.translatingPart ~= nil then
                        object:addWashableNode(input.translatingPart);
                    end;
                    object:setDirtAmount(object:getDirtAmount(), true);
                end;
            end;
        end;
    end
    AttacherJoints.attachImplement = Utils.appendedFunction(AttacherJoints.attachImplement, AttachImplementAppendedFunction);

    function AttacherJoints:detachImplement(superFunc, implementIndex, noEventSend)
        if superFunc ~= nil then
            if not superFunc(self, implementIndex, noEventSend) then
                return false;
            end
        end

        if noEventSend == nil or noEventSend == false then
            if g_server ~= nil then
                g_server:broadcastEvent(VehicleDetachEvent:new(self, self.attachedImplements[implementIndex].object), nil, nil, self);
            else
                -- Send detach request to server and return
                local implement = self.attachedImplements[implementIndex];
                if implement.object ~= nil then
                    g_client:getServerConnection():sendEvent(VehicleDetachEvent:new(self, implement.object));
                end
                return;
            end
        end

        self:onDetachImplement(implementIndex);

        local implement = self.attachedImplements[implementIndex];
        local jointDesc;
        if implement.object ~= nil then
            jointDesc = self.attacherJoints[implement.jointDescIndex];
            if jointDesc.transNode ~= nil then
                setTranslation(jointDesc.transNode, unpack(jointDesc.transNodeOrgTrans));
            end
            if not implement.object.isHardAttached then
                if self.isServer then
                    if jointDesc.jointIndex ~= 0 then
                        removeJoint(jointDesc.jointIndex);
                    end
                end
            end
            jointDesc.jointIndex = 0;
        end

        ObjectChangeUtil.setObjectChanges(jointDesc.changeObjects, false, self, self.setMovingToolDirty);

        local rootAttacherVehicle = self:getRootAttacherVehicle();
        local selectNewImplement = false;

        if self.isClient then
            if rootAttacherVehicle.selectedImplement == implement then
                selectNewImplement = true;
                rootAttacherVehicle:setSelectedImplement(nil);
            end
        end
        if implement.object ~= nil then
            local object = implement.object;
            implement.object:onDetach(self, implement.jointDescIndex);
            if implement.object.isHardAttached then
                self:hardDetachImplement(implement)
            end
            implement.object = nil;
            if self.isClient then
                if jointDesc.topArm ~= nil then
                    setRotation(jointDesc.topArm.rotationNode, jointDesc.topArm.rotX, jointDesc.topArm.rotY, jointDesc.topArm.rotZ);
                    if jointDesc.topArm.translationNode ~= nil then
                        setTranslation(jointDesc.topArm.translationNode, 0, 0, 0);
                    end
                    if jointDesc.topArm.scaleNode ~= nil then
                        setScale(jointDesc.topArm.scaleNode, 1, 1, 1);
                    end
                    if jointDesc.topArm.toggleVisibility then
                        setVisibility(jointDesc.topArm.rotationNode, false);
                    end
                end
                if jointDesc.bottomArm ~= nil then
                    setRotation(jointDesc.bottomArm.rotationNode, jointDesc.bottomArm.rotX, jointDesc.bottomArm.rotY, jointDesc.bottomArm.rotZ);
                    if jointDesc.bottomArm.translationNode ~= nil then
                        setTranslation(jointDesc.bottomArm.translationNode, 0, 0, 0);
                    end
                    if self.setMovingToolDirty ~= nil then
                        self:setMovingToolDirty(jointDesc.bottomArm.rotationNode);
                    end
                    if jointDesc.bottomArm.toolbar ~= nil then
                        setVisibility(jointDesc.bottomArm.toolbar, false)
                    end
                end
            end
            if self.isServer then
                -- restore original translation
                setTranslation(jointDesc.jointTransform, unpack(jointDesc.jointOrigTrans));
                setTranslation(object.attacherJoint.node, unpack(object.attacherJoint.jointOrigTrans));

                if jointDesc.rotationNode ~= nil then
                    setRotation(jointDesc.rotationNode, jointDesc.rotX, jointDesc.rotY, jointDesc.rotZ);
                end
            end

            local unlinkPto = function(object, jointDesc, name)
                if jointDesc[name.."Active"] then
                    local input = object[name.."Input"]
                    local jointDescOutput = jointDesc[name.."Output"]

                    if input.rootNode ~= nil then
                        unlink(input.rootNode);
                        unlink(input.attachNode);

                        if object.removeWashableNode ~= nil then
                            object:removeWashableNode(input.rootNode);
                            object:removeWashableNode(input.attachNode);
                            object:removeWashableNode(input.dirAndScaleNode);
                            if input.translatingPart ~= nil then
                                object:removeWashableNode(input.translatingPart);
                            end;
                        end
                    else
                        unlink(jointDescOutput.rootNode);
                        unlink(jointDescOutput.attachNode);

                        if object.removeWashableNode ~= nil then
                            object:removeWashableNode(jointDescOutput.rootNode);
                            object:removeWashableNode(jointDescOutput.attachNode);
                            object:removeWashableNode(jointDescOutput.dirAndScaleNode);
                        end
                    end
                    jointDesc[name.."Active"] = false;
                end
            end

            unlinkPto(object, jointDesc, "pto")
            unlinkPto(object, jointDesc, "pto2")


            object:onDetached(self, implement.jointDescIndex);
        end

        table.remove(self.attachedImplements, implementIndex);

        if self.isClient then
            if selectNewImplement then
                local newSelectedImplement = nil;
                local newIndex = math.min(implementIndex, table.getn(self.attachedImplements));
                if newIndex == 0 then
                    if self ~= rootAttacherVehicle then
                        -- select self
                        newSelectedImplement = self.attacherVehicle:getImplementByObject(self);
                    end
                else
                    -- select the implement at the new index
                    newSelectedImplement = self.attachedImplements[newIndex];
                end
                rootAttacherVehicle:setSelectedImplement(newSelectedImplement);
            end
        end

        self:playDetachSound(jointDesc);

        return true;
    end
        
    function AttacherJoints:updatePowerTakeoff(superFunc, implement, dt, name)
        if superFunc ~= nil then
            superFunc(self, implement, dt, name);
        end;
        
        local jointDesc = self.attacherJoints[implement.jointDescIndex];
        if jointDesc[name.."Active"] then
            local object = implement.object;

            local output = jointDesc[name.."Output"];
            local input = object[name.."Input"];

            local ptoRootNode = output.rootNode;
            local ptoAttachNode = output.attachNode;
            local ptoTranslatingPart = nil;
            local ptoDirAndScaleNode = output.dirAndScaleNode;
            local ptoBaseDistance = output.baseDistance;
            
            if input.rootNode ~= nil then
                ptoRootNode = input.rootNode;
                ptoAttachNode = input.attachNode;
                ptoTranslatingPart = input.translatingPart;
                ptoDirAndScaleNode = input.dirAndScaleNode;
                ptoBaseDistance = input.baseDistance;
            end;

            if input.rotSpeed ~= 0 and object:getIsPowerTakeoffActive() then
                output.rotation = (output.rotation + dt*input.rotSpeed) % (2*math.pi);
                setRotation(ptoRootNode, 0, 0, output.rotation);
                setRotation(ptoAttachNode, 0, 0, output.rotation);
            end;

            local x,y,z = getWorldTranslation(ptoAttachNode);
            local dx,dy,dz = worldToLocal(ptoRootNode, x,y,z);
            setDirection(ptoDirAndScaleNode, dx, dy, dz, 0, 1, 0);
            
            if ptoTranslatingPart == nil then
                local _,_,dist = worldToLocal(ptoDirAndScaleNode, x,y,z);
                setScale(ptoDirAndScaleNode, 1, 1, dist/ptoBaseDistance);
            else
                local _,_,dist = worldToLocal(ptoDirAndScaleNode, x,y,z);
                setTranslation(ptoTranslatingPart, 0, 0, dist-ptoBaseDistance);
            end;
        end;
    end;
end;