--[[************************************************************************************************************************************************************
    
    Override Game Scripts for Fillable Extension
    
    31.10.2018 © Saty MODS (SatyMODS / http://forum.lsczech.cz)
    
****************************************************************************************************************************************************************
    
    (EN) INFO
    
    Free for use on mods. Modifications only with my permission !
    
****************************************************************************************************************************************************************
    
    (CZ) INFO
    
    Pro volne pouziti v modech. ZAKAZ modifikaci bez meho souhlasu !
    
************************************************************************************************************************************************************]]--

if not BaseMission.SatyMODShasOverrideScripts then
    BaseMission.SatyMODShasOverrideScripts = true;

    
    local StoreItemsUtilAddStoreItem = StoreItemsUtil.addStoreItem;
    function StoreItemsUtil.addStoreItem(name, functions, specs, imageActive, price, dailyUpkeep, runningLeasingFactor, lifetime, incomePerHour, xmlFilename, species, canBeSold, configurations, rotation, brand, category, fieldJobData, isMod, dlcTitle, customEnvironment, achievementsNeeded, sharedVramUsage, perInstanceVramUsage, ignoreVramUsage, bundleInfo, isBundleItem)
        local item = StoreItemsUtilAddStoreItem(name, functions, specs, imageActive, price, dailyUpkeep, runningLeasingFactor, lifetime, incomePerHour, xmlFilename, species, canBeSold, configurations, rotation, brand, category, fieldJobData, isMod, dlcTitle, customEnvironment, achievementsNeeded, sharedVramUsage, perInstanceVramUsage, ignoreVramUsage, bundleInfo, isBundleItem);
        
        if isMod and configurations ~= nil then
            local xmlFile = loadXMLFile("storeItemXML", xmlFilename);
            
            local storeImageOriginal = getXMLString(xmlFile, "vehicle.storeData.image");
            local overrideImagePath = getXMLBool(xmlFile, "vehicle.storeData.overrideImagePath");

            
            if overrideImagePath and storeImageOriginal ~= nil and storeImageOriginal ~= "" then
                local baseDir = string.sub(imageActive, 1, string.find(imageActive, storeImageOriginal) - 1);
                
                if configurations["fillableExtension"] ~= nil then
                    item.fillableExtensions = {};
                    item.fillableExtensionsCapacity = {};
                    item.fillableExtensionsFillType = {};
                    item.fillableExtensionsFillInfo = {};
                    
                    local i = 0;
                    while true do
                        local functionKey = string.format("vehicle.fillableExtensionConfigurations.fillableExtensionConfiguration(%d)", i);
                        if not hasXMLProperty(xmlFile, functionKey) then
                            break;
                        end;
                        
                        local fillableExtension = {};
                        
                        local j = 0;
                        while true do
                            local colorKey = string.format(functionKey .. ".baseColorConfigurations.baseColorConfiguration(%d)", j);
                            if not hasXMLProperty(xmlFile, colorKey) then
                                break;
                            end;
                            
                            local entry = {};
                            entry.color = Utils.getVectorNFromString(getXMLString(xmlFile, colorKey .. "#color"), 4);
                            entry.storeImage = imageActive;
                            
                            local img = getXMLString(xmlFile, colorKey .. "#storeImage")
                            if img ~= nil then
                                entry.storeImage = Utils.getFilename(img, baseDir);
                            end;
                            
                            table.insert(fillableExtension, entry);
                            j = j + 1;
                        end;
                        
                        table.insert(item.fillableExtensions, fillableExtension);
                        
                        local fillTypeName = getXMLString(xmlFile, functionKey .. ".fillType#name");
                        if fillTypeName == nil then
                            fillTypeName = getXMLString(xmlFile, functionKey .. ".fillUnits.fillUnit(0)#fillTypeCategories");
                        end;
                        if fillTypeName ~= nil then
                            item.fillableExtensionsFillType[i+1] = fillTypeName;
                        end;
                        
                        local fillTypeInfo = getXMLString(xmlFile, functionKey .. ".fillType#info");
                        if fillTypeInfo ~= nil then
                            item.fillableExtensionsFillInfo[i+1] = fillTypeInfo;
                        end;
                        
                        local capacity = getXMLFloat(xmlFile, functionKey .. ".fillUnits.fillUnit(0)#capacity");
                        if capacity ~= nil and capacity > 0 then
                            item.fillableExtensionsCapacity[i+1] = capacity;
                        end;
                        
                        i = i + 1;
                    end;
                end;
                
                if configurations["balersExtension"] ~= nil then
                    item.balersExtensions = {};
                    
                    local i = 0;
                    while true do
                        local functionKey = string.format("vehicle.balersExtensionConfigurations.balersExtensionConfiguration(%d)", i);
                        if not hasXMLProperty(xmlFile, functionKey) then
                            break;
                        end;
                        
                        local balersExtension = {};
                        
                        local j = 0;
                        while true do
                            local colorKey = string.format(functionKey .. ".baseColorConfigurations.baseColorConfiguration(%d)", j);
                            if not hasXMLProperty(xmlFile, colorKey) then
                                break;
                            end;
                            
                            local entry = {};
                            entry.color = Utils.getVectorNFromString(getXMLString(xmlFile, colorKey .. "#color"), 4);
                            entry.storeImage = imageActive;
                            
                            local img = getXMLString(xmlFile, colorKey .. "#storeImage")
                            if img ~= nil then
                                entry.storeImage = Utils.getFilename(img, baseDir);
                            end;
                            
                            table.insert(balersExtension, entry);
                            j = j + 1;
                        end;
                        
                        table.insert(item.balersExtensions, balersExtension);
                        
                        i = i + 1;
                    end;
                end;
                
                if configurations["baseColor"] ~= nil and configurations["fillableExtension"] == nil and configurations["balersExtension"] == nil then
                    item.baseColorImages = {};
                    
                    local i = 0;
                    while true do
                        local colorKey = string.format("vehicle.baseColorConfigurations.baseColorConfiguration(%d)", i);
                        if not hasXMLProperty(xmlFile, colorKey) then
                            break;
                        end;
                        
                        local storeImage = imageActive;
                        local img = getXMLString(xmlFile, colorKey .. "#storeImage")
                        if img ~= nil then
                            storeImage = Utils.getFilename(img, baseDir);
                        end;
                        
                        table.insert(item.baseColorImages, storeImage);
                        i = i + 1;
                    end;
                end;
                
                if configurations["rimColor"] ~= nil then
                    local wheelsFilePath = Utils.getFilename("IMG/store_wheels.dds", baseDir);
                    if fileExists(wheelsFilePath) then
                        item.wheelsImage = wheelsFilePath;
                    end;
                end;
            end;
            
            delete(xmlFile);
        end;
        
        return item
    end;
    
    
    g_shopConfigScreenUpdateData = function(self)
        if self.storeItem.fillableExtensions ~= nil then
            local fillExt = self.storeItem.fillableExtensions[self.configurations["fillableExtension"]];
            if fillExt ~= nil then
                local id = self.configurations["baseColor"];
                
                self.shopConfigItemIcon:setImageFilename(fillExt[id].storeImage);
                
                for i, value in pairs(fillExt) do
                    self.storeItem.configurations["baseColor"][i].color = value.color;
                end;
                
                self.configColorPreviewMapping["baseColor"]:setImageColor(nil, unpack(self.storeItem.configurations["baseColor"][id].color));
            end;
            
            local capacity = self.storeItem.fillableExtensionsCapacity[self.configurations["fillableExtension"]];
            if capacity ~= nil then
                self.attrIcon[2]:applyProfile("shopListAttributeIconCapacity");
                self.attrValue[2]:setText(string.format(capacity .. " l"));
                self.attrIcon[2]:setVisible(true);
                self.attrValue[2]:setVisible(true);
            end;
            
            local fillTypeName = self.storeItem.fillableExtensionsFillType[self.configurations["fillableExtension"]];
            if fillTypeName ~= nil then
                self.attrIcon[3]:applyProfile("shopListAttributeIconFillTypes");
                self.attrValue[3]:setText(fillTypeName);
                self.attrIcon[3]:setVisible(true);
                self.attrValue[3]:setVisible(true);
            end;
            
            local fillTypeInfo = self.storeItem.fillableExtensionsFillInfo[self.configurations["fillableExtension"]];
            if fillTypeInfo ~= nil then
                self.attrIcon[4]:applyProfile("shopListAttributeIconInfo");
                self.attrValue[4]:setText(fillTypeInfo);
                self.attrIcon[4]:setVisible(true);
                self.attrValue[4]:setVisible(true);
            end;
        end;
        
        if self.storeItem.balersExtensions ~= nil then
            local fillExt = self.storeItem.balersExtensions[self.configurations["balersExtension"]];
            if fillExt ~= nil then
                local id = self.configurations["baseColor"];
                
                self.shopConfigItemIcon:setImageFilename(fillExt[id].storeImage);
                
                for i, value in pairs(fillExt) do
                    self.storeItem.configurations["baseColor"][i].color = value.color;
                end;
                
                self.configColorPreviewMapping["baseColor"]:setImageColor(nil, unpack(self.storeItem.configurations["baseColor"][id].color));
            end;
        end;

        if self.storeItem.baseColorImages ~= nil then
            local id = self.configurations["baseColor"];
            if self.vehicle ~= nil then
                id = self.vehicle.configurations["baseColor"];
            end;
                
            self.shopConfigItemIcon:setImageFilename(self.storeItem.baseColorImages[id]);
        end;
        
        if self.storeItem.wheelsImage ~= nil then
            if self.shopConfigItemIconWheels == nil then
                self.shopConfigItemIconWheels = self.shopConfigItemIcon:clone(g_shopConfigScreen);
            end;
            if self.storeItem.configurations["rimColor"][self.configurations["rimColor"]] ~= nil then
                local r,g,b,a = unpack(self.storeItem.configurations["rimColor"][self.configurations["rimColor"]].color);
                self.shopConfigItemIconWheels:setImageFilename(self.storeItem.wheelsImage);
                self.shopConfigItemIconWheels:setAlpha(1);
                self.shopConfigItemIconWheels:setImageColor(nil, r, g, b, a);
            end;
        else
            if self.shopConfigItemIconWheels ~= nil then
                self.shopConfigItemIconWheels:setAlpha(0);
            end;
        end;
    end;
    g_shopConfigScreen.updateData = Utils.appendedFunction(g_shopConfigScreen.updateData, g_shopConfigScreenUpdateData);
    

    DirectSellDialogSetVehicle = function(self, vehicle, owner, ownWorkshop)
        if vehicle ~= nil then
            if self.storeItem.fillableExtensions ~= nil then
                local fillExt = self.storeItem.fillableExtensions[vehicle.configurations["fillableExtension"]];
                if fillExt ~= nil then
                    local id = vehicle.configurations["baseColor"];
                    
                    self.dialogImage:setImageFilename(fillExt[id].storeImage);
                end;
            end;
            
            if self.storeItem.balersExtensions ~= nil then
                local fillExt = self.storeItem.balersExtensions[vehicle.configurations["balersExtension"]];
                if fillExt ~= nil then
                    local id = vehicle.configurations["baseColor"];
                    
                    self.dialogImage:setImageFilename(fillExt[id].storeImage);
                end;
            end;
            
            if self.storeItem.baseColorImages ~= nil then
                local id = vehicle.configurations["baseColor"];
                
                self.dialogImage:setImageFilename(self.storeItem.baseColorImages[id]);
            end;
            
            if self.storeItem.wheelsImage ~= nil then
                if self.dialogImageWheels == nil then
                    self.dialogImageWheels = self.dialogImage:clone(g_directSellDialog);
                end;
                if self.storeItem.configurations["rimColor"][vehicle.configurations["rimColor"]] ~= nil then
                    local r,g,b,a = unpack(self.storeItem.configurations["rimColor"][vehicle.configurations["rimColor"]].color);
                    self.dialogImageWheels:setImageFilename(self.storeItem.wheelsImage);
                    self.dialogImageWheels:setAlpha(1);
                    self.dialogImageWheels:setImageColor(nil, r, g, b, a);
                end;
            else
                if self.dialogImageWheels ~= nil then
                    self.dialogImageWheels:setAlpha(0);
                end;
            end;
        end;
    end;
    DirectSellDialog.setVehicle = Utils.appendedFunction(DirectSellDialog.setVehicle, DirectSellDialogSetVehicle);
    
    
    local InGameMenuGetHotspotData = InGameMenu.getHotspotData;
    function InGameMenu:getHotspotData(hotspot)
        local name, imageActive, UVs, vehicle = InGameMenuGetHotspotData(self, hotspot);
    
        if vehicle ~= nil then
            local storeItem = StoreItemsUtil.storeItemsByXMLFilename[vehicle.configFileName:lower()]
            if storeItem ~= nil then
                if storeItem.fillableExtensions ~= nil then
                    local fillExt = storeItem.fillableExtensions[vehicle.configurations["fillableExtension"]];
                    if fillExt ~= nil then
                        local id = vehicle.configurations["baseColor"];
                        
                        imageActive = fillExt[id].storeImage;
                    end;
                end;
                
                if storeItem.balersExtensions ~= nil then
                    local fillExt = storeItem.balersExtensions[vehicle.configurations["balersExtension"]];
                    if fillExt ~= nil then
                        local id = vehicle.configurations["baseColor"];
                        
                        imageActive = fillExt[id].storeImage;
                    end;
                end;
                
                if storeItem.baseColorImages ~= nil then
                    local id = vehicle.configurations["baseColor"];
                    
                    imageActive = storeItem.baseColorImages[id];
                end;
            end;
        end;

        return name, imageActive, UVs, vehicle;
    end;
    
    
    SellItemDialogSetItem = function(self, item, price)
        local imageActive = "dataS2/menu/blank.png";
        if item ~= nil then
            local storeItem = StoreItemsUtil.storeItemsByXMLFilename[item.configFileName:lower()];
            if storeItem ~= nil then
                imageActive = storeItem.imageActive;
                
                if storeItem.fillableExtensions ~= nil then
                    local fillExt = storeItem.fillableExtensions[item.configurations["fillableExtension"]];
                    if fillExt ~= nil then
                        local id = item.configurations["baseColor"];
                        imageActive = fillExt[id].storeImage;
                    end;
                end;
                
                if storeItem.balersExtensions ~= nil then
                    local fillExt = storeItem.balersExtensions[item.configurations["balersExtension"]];
                    if fillExt ~= nil then
                        local id = item.configurations["baseColor"];
                        imageActive = fillExt[id].storeImage;
                    end;
                end;
                
                if storeItem.baseColorImages ~= nil then
                    local id = item.configurations["baseColor"];
                    
                    imageActive = storeItem.baseColorImages[id];
                end;
                
                if storeItem.wheelsImage ~= nil then
                    if self.dialogImageElementWheels == nil then
                        self.dialogImageElementWheels = self.dialogImageElement:clone(g_sellItemDialog);
                    end;
                    if storeItem.configurations["rimColor"][item.configurations["rimColor"]] ~= nil then
                        local r,g,b,a = unpack(storeItem.configurations["rimColor"][item.configurations["rimColor"]].color);
                        self.dialogImageElementWheels:setImageFilename(storeItem.wheelsImage);
                        self.dialogImageElementWheels:setAlpha(1);
                        self.dialogImageElementWheels:setImageColor(nil, r, g, b, a);
                    end;
                else
                    if self.dialogImageElementWheels ~= nil then
                        self.dialogImageElementWheels:setAlpha(0);
                    end;
                end;
            end;
        end;
        self.dialogImageElement:setImageFilename(imageActive)
    end;
    SellItemDialog.setItem = Utils.appendedFunction(SellItemDialog.setItem, SellItemDialogSetItem);
    
    
    function ShopScreen:showItemOverview(items, page, filter, isRealObj)
        self:setSliderController(nil);
        self.currentPageElement = self.pageVehicles;
        self.currentPageElement:setVisible(true);
        FocusManager:setFocus(self.itemListElement);

        self.currentPriceList = {};
        self.currentItemList = {};
        self.currentRealItemList = {};
        self.itemListElement:setVisible(false);
        self.itemListElement:deleteListItems();
        self.currentItemIndex = nil;
        self.currentFilter = filter;
        self.itemListElement.ignoreUpdate = true;
        for _, item in pairs(items) do
            self.currentItem = item;
            self.currentRealItem = item;
            if isRealObj then
                self.currentItem = StoreItemsUtil.storeItemsByXMLFilename[item.configFileName:lower()];
                
                if self.currentItem.fillableExtensions ~= nil then
                    self.currentItem.imageActiveOld = self.currentItem.imageActive;
                    
                    local fillExt = self.currentItem.fillableExtensions[item.configurations["fillableExtension"]];
                    if fillExt ~= nil then
                        local id = item.configurations["baseColor"];
                        self.currentItem.imageActive = fillExt[id].storeImage;
                    end;
                    
                    self.vehicleItemTemplate:updateAbsolutePosition();
                end;
                
                if self.currentItem.balersExtensions ~= nil then
                    self.currentItem.imageActiveOld = self.currentItem.imageActive;
                    
                    local fillExt = self.currentItem.balersExtensions[item.configurations["balersExtension"]];
                    if fillExt ~= nil then
                        local id = item.configurations["baseColor"];
                        self.currentItem.imageActive = fillExt[id].storeImage;
                    end;
                    
                    self.vehicleItemTemplate:updateAbsolutePosition();
                end;
                
                if self.currentItem.baseColorImages ~= nil then
                    self.currentItem.imageActiveOld = self.currentItem.imageActive;
                    
                    local id = item.configurations["baseColor"];
                    self.currentItem.imageActive = self.currentItem.baseColorImages[id];
                    
                    self.vehicleItemTemplate:updateAbsolutePosition();
                end;
            end;
            local newListItem = self.vehicleItemTemplate:clone(self.itemListElement);
            newListItem:updateAbsolutePosition();
            table.insert(self.currentItemList, self.currentItem);
            table.insert(self.currentRealItemList, self.currentRealItem);
            if self.currentItem.imageActiveOld ~= nil then
                self.currentItem.imageActive = self.currentItem.imageActiveOld;
                self.currentItem.imageActiveOld = nil;
            end;
            self.currentItem = nil;
            self.currentRealItem = nil;
        end;

        self.shopListSeparator1:setVisible(#self.currentItemList > 0);
        self.shopListSeparator2:setVisible(#self.currentItemList > 1);
        self.shopListSeparator3:setVisible(#self.currentItemList > 2);

        self.itemListElement.ignoreUpdate = false;
        self.itemListElement:updateItemPositions();
        self.itemListElement:setVisible(true);
        local numVehicles = table.getn(self.currentItemList);

        self:setSliderController(self.itemListElement);
        
        if numVehicles > 0 then
            local selectedIndex, firstVisibleItem = self:getLastSelection(page, filter);
            self.selectedIndex = selectedIndex;
            self.itemListElement:scrollTo(firstVisibleItem, false);
            self.itemListElement:setSelectedRow(selectedIndex, true);
        else
            self:updateAttributes();
        end;

        self.noVehiclesAvailable:setVisible(numVehicles == 0);
        self:updatePageTitles();
    end;
end;
